@@ -12,17 +12,17 @@
  *  codec.
  *	
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
@@ -31,7 +31,7 @@
 #ifndef DV_BITSTREAM_H
 #define DV_BITSTREAM_H
 
-#include <dv_types.h>
+#include "dv_types.h"
 #if HAVE_ENDIAN_H
 #include <endian.h>
 #elif HAVE_MACHINE_ENDIAN_H
@@ -52,17 +52,17 @@ extern "C" {
  (((uint8_t*)&x)[2] << 8)  | (((uint8_t*)&x)[3]))
 #endif // LITTLE_ENDIAN
 
-bitstream_t *bitstream_init();
-void bitstream_set_fill_func(bitstream_t *bs,uint32_t (*next_function) (uint8_t **,void *),void *priv);
-void bitstream_next_buffer(bitstream_t * bs);
-void bitstream_new_buffer(bitstream_t *bs,uint8_t *buf,uint32_t len);
-void bitstream_byte_align(bitstream_t *bs);
+bitstream_t *_dv_bitstream_init();
+void _dv_bitstream_set_fill_func(bitstream_t *bs,uint32_t (*next_function) (uint8_t **,void *),void *priv);
+void _dv_bitstream_next_buffer(bitstream_t * bs);
+void _dv_bitstream_new_buffer(bitstream_t *bs,uint8_t *buf,uint32_t len);
+void _dv_bitstream_byte_align(bitstream_t *bs);
 
 static void bitstream_next_word(bitstream_t *bs) {
   uint32_t diff = bs->buflen - bs->bufoffset;
 
   if (diff == 0)
-    bitstream_next_buffer(bs);
+    _dv_bitstream_next_buffer(bs);
 
   if ((bs->buflen - bs->bufoffset) >=4 ) {
     bs->next_word = *(uint32_t *)(bs->buf + bs->bufoffset);
@@ -74,7 +74,7 @@ static void bitstream_next_word(bitstream_t *bs) {
     bs->next_word = *(uint32_t *)(bs->buf + bs->buflen - 4);
     bs->next_bits = (bs->buflen - bs->bufoffset) * 8;
 //    fprintf(stdout,"end of buffer, have %d bits\n",bs->next_bits);
-    bitstream_next_buffer(bs);
+    _dv_bitstream_next_buffer(bs);
 //    fprintf(stderr,"next_word is %08x at %d\n",bs->next_word,bs->bufoffset);
   }
 }
