@@ -115,7 +115,7 @@ int _dv_raw_insert_audio(unsigned char * frame_buf,
 {
 	int dif_seg;
 	int dif_seg_max = isPAL ? 12 : 10;
-	int samplesperframe = audio->bytesperframe;
+	int samplesperframe = audio->bytesperframe/(2*audio->channels);
 	
 	int bits_per_sample = 16;
 	unsigned char* wave_buf = audio->data;
