@@ -8,27 +8,31 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
  */
 
 
-#include <dv_types.h>
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <math.h>
 
+#include "rgb.h"
+
 #define COLOR_FRACTION_BITS 10
 #define COLOR_FRACTION_MUL  (1 << COLOR_FRACTION_BITS)
 
@@ -254,14 +258,14 @@ dv_mb420_rgb(dv_macroblock_t *mb, uint8_t **pixels, int *pitches) {
 
   for (j = 0; j < 4; j += 2) { // Two rows of blocks j, j+1
 
-    for (row = 0; row < 8; row+=2) { // 4 pairs of two rows
+    for (row = 0; row < 4; row++) { // 4 pairs of two rows
       pwrgb0 = prgb;
-      pwrgb1 = prgb + pitches[0];
+      pwrgb1 = prgb + pitches[0] * 2;
 
       for (i = 0; i < 2; ++i) { // Two columns of blocks
         int yindex = j + i;
         dv_coeff_t *Ytmp0 = Y[yindex];
-        dv_coeff_t *Ytmp1 = Y[yindex] + 8;
+        dv_coeff_t *Ytmp1 = Y[yindex] + 16;
         for (col = 0; col < 4; ++col) {  // 4 spans of 2x2 pixels
           int8_t cb = clamp (-128, *cb_frame++, 127); // +128;
           int8_t cr = clamp (-128, *cr_frame++, 127); // +128
@@ -288,10 +292,12 @@ dv_mb420_rgb(dv_macroblock_t *mb, uint8_t **pixels, int *pitches) {
           } // for k
 
         } // for col
-        Y[yindex] = Ytmp1;
+        if (row & 1)
+          Ytmp0 += 16;
+        Y[yindex] = Ytmp0;
       } // for i
 
-      prgb += (2 * pitches[0]);
+      prgb += (row & 1) ? 3 * pitches [0] : pitches [0];
     } // for row
 
   } // for j
@@ -427,14 +433,14 @@ dv_mb420_bgr0(dv_macroblock_t *mb, uint8_t **pixels, int *pitches) {
 
   for (j = 0; j < 4; j += 2) { // Two rows of blocks j, j+1
 
-    for (row = 0; row < 8; row+=2) { // 4 pairs of two rows
+    for (row = 0; row < 4; row++) { // 4 pairs of two rows
       pwrgb0 = prgb;
-      pwrgb1 = prgb + pitches[0];
+      pwrgb1 = prgb + pitches[0] * 2;
 
       for (i = 0; i < 2; ++i) { // Two columns of blocks
         int yindex = j + i;
         dv_coeff_t *Ytmp0 = Y[yindex];
-        dv_coeff_t *Ytmp1 = Y[yindex] + 8;
+        dv_coeff_t *Ytmp1 = Y[yindex] + 16;
         for (col = 0; col < 4; ++col) {  // 4 spans of 2x2 pixels
           int8_t cb = clamp (-128, *cb_frame++, 127); // +128;
           int8_t cr = clamp (-128, *cr_frame++, 127); // +128
@@ -463,10 +469,12 @@ dv_mb420_bgr0(dv_macroblock_t *mb, uint8_t **pixels, int *pitches) {
           } // for k
 
         } // for col
-        Y[yindex] = Ytmp1;
+        if (row & 1)
+          Ytmp0 += 16;
+        Y[yindex] = Ytmp0;
       } // for i
 
-      prgb += (2 * pitches[0]);
+      prgb += (row & 1) ? 3 * pitches [0] : pitches [0];
     } // for row
 
   } // for j
