@@ -8,17 +8,17 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
@@ -27,6 +27,9 @@
 /* Most of this file is derived from patches 101018 and 101136 submitted by
  * Stefan Lucke <Stefan.Lucke1@epost.de> */
 
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -34,8 +37,8 @@
 #include <math.h>
 #include <string.h>
 
-#include <libdv/dv_types.h>
-#include <libdv/util.h>
+#include "libdv/dv_types.h"
+#include "libdv/util.h"
 #include "display.h"
 
 #if HAVE_LIBXV
@@ -91,6 +94,7 @@ dv_display_popt_callback(poptContext con, enum poptCallbackReason reason,
     dv_opt_usage(con, display->option_table, DV_DISPLAY_OPT_METHOD);
   } /* if */
 
+#if HAVE_LIBXV
   if (display->arg_aspect_string) {
     if (strlen (display->arg_aspect_string) == 1) {
       switch (display->arg_aspect_string[0]) {
@@ -117,6 +121,7 @@ dv_display_popt_callback(poptContext con, enum poptCallbackReason reason,
       ((display->arg_size_val < 10) || (display->arg_size_val > 100))) {
     dv_opt_usage(con, display->option_table, DV_DISPLAY_OPT_SIZE);
   } /* if */
+#endif /* HAVE_LIBXV */
 
 } /* dv_display_popt_callback */
 #endif /* HAVE_LIBPOPT */
@@ -139,6 +144,7 @@ dv_display_new(void)
     argDescrip: "(0|1|2|3)",
 		  }; /* display method */
 
+#if HAVE_LIBXV
   result->option_table[DV_DISPLAY_OPT_ASPECT] = (struct poptOption) {
     longName:   "aspect",
     argInfo:    POPT_ARG_STRING,
@@ -157,12 +163,6 @@ dv_display_new(void)
     argDescrip: "(10 .. 100)",
 		  }; /* display method */
 
-  result->option_table[DV_DISPLAY_OPT_CALLBACK] = (struct poptOption){
-    argInfo: POPT_ARG_CALLBACK|POPT_CBFLAG_POST,
-    arg:     dv_display_popt_callback,
-	       descrip: (char *)result, /* data passed to callback */
-	       }; /* callback */
-
   result->option_table[DV_DISPLAY_OPT_XV_PORT] = (struct poptOption) {
     longName:   "xvport", 
     argInfo:    POPT_ARG_INT, 
@@ -171,6 +171,15 @@ dv_display_new(void)
     descrip:    "set Xvideo port (defaults to the first usable)",
   }; /* choose Xvideo port */
 
+#endif /* HAVE_LIBXV */
+
+  result->option_table[DV_DISPLAY_OPT_CALLBACK] = (struct poptOption){
+    argInfo: POPT_ARG_CALLBACK|POPT_CBFLAG_POST,
+    arg:     dv_display_popt_callback,
+	       descrip: (char *)result, /* data passed to callback */
+	       }; /* callback */
+
+
 #endif /* HAVE_LIBPOPT */
 
  no_mem:
@@ -576,11 +585,11 @@ dv_display_Xv_init(dv_display_t *dv_dpy, gchar *w_name, gchar *i_name,
     shmat(dv_dpy->shminfo.shmid, 0, 0);
 
   XShmAttach(dv_dpy->dpy, &dv_dpy->shminfo);
+  XSync(dv_dpy->dpy, False);
+
   if (dv_dpy -> shminfo. shmid > 0)
     shmctl (dv_dpy -> shminfo. shmid, IPC_RMID, 0);
 
-  XSync(dv_dpy->dpy, False);
-
   return 1;
 } /* dv_display_Xv_init */
 #endif /* HAVE_LIBXV */
@@ -744,7 +753,7 @@ dv_display_init(dv_display_t *dv_dpy, gint *argc, gchar ***argv, gint width, gin
       goto fail;
     }
 #else /* HAVE_LIBXV */
-    fprintf(stderr, "Attempt to display via Xv failed\n");
+    fprintf(stderr, "Playdv not compiled with Xv support, sorry.\n");
     goto fail;
 #endif /* HAVE_LIBXV */
     break;
